@@ -6,6 +6,7 @@ import sys
 from typing import List, Tuple, Optional, Dict, Sequence, Iterable
 from pathlib import Path
 from datetime import datetime
+from subprocess import check_call
 from tempfile import TemporaryDirectory
 
 
@@ -13,7 +14,7 @@ from . import config
 from . import server
 from .misc import install_server
 from .common import PathIsh, History, make_filter, get_logger, get_tmpdir
-from .common import previsits_to_history, Source
+from .common import previsits_to_history, Source, appdirs
 from .dump import dump_histories
 
 
@@ -35,25 +36,23 @@ def _do_index() -> Iterable[Exception]:
 
 
     all_histories = []
-    all_errors = []
+    errors = []
 
-    for extractor in indexers:
-        ex = extractor
-        if callable(ex):
-            # lazy indexers
-            ex = ex()
-        assert isinstance(ex, Source)
+    for idx in indexers:
+        if isinstance(idx, Exception):
+            errors.append(idx)
+            continue
+        # TODO more defensive! e.g. might not have __module__
+        einfo = f'{idx.ff.__module__}:{idx.ff.__name__} {idx.args} {idx.kwargs}'
 
-        einfo = f'{ex.ff.__module__}:{ex.ff.__name__} {ex.args} {ex.kwargs}'
-
-        hist, errors = previsits_to_history(ex, src=ex.src)
-        all_errors.extend(errors)
+        hist, err = previsits_to_history(idx, src=idx.name)
+        errors.extend(err)
         all_histories.append((einfo, hist))
 
     # TODO perhaps it's better to open connection and dump as we collect so it consumes less memory?
     dump_histories(all_histories)
 
-    return all_errors
+    return errors
 
 
 def do_index(config_file: Path):
@@ -120,27 +119,105 @@ def do_demo(*, index_as: str, params: Sequence[str], port: Optional[str], config
             input("Press any key when ready")
 
 
+def user_config_file() -> Path:
+    return Path(appdirs().user_config_dir) / 'config.py'
+
+
+def default_config_path() -> Path:
+    cfg = Path('config.py')
+    if cfg.exists():
+        # eh. not sure if it's a good idea, but whatever, it was the old behaviour
+        return cfg.absolute()
+    else:
+        return user_config_file()
+    # TODO need to test this..
+
+
+def config_create(args):
+    logger = get_logger()
+    cfg = user_config_file()
+    cfgdir = cfg.parent
+    if cfgdir.exists():
+        logger.error('Config directory %s already exists. Aborting', cfgdir)
+        sys.exit(1)
+    else:
+        # TODO ugh. example config might not be in the repository
+        stub = """
+from promnesia import Source
+from promnesia.sources import auto
+
+'''
+List of sources to use
+You can specify your own, add more sources, etc.
+See https://github.com/karlicoss/promnesia#setup for more information
+'''
+SOURCES = [
+    Source(
+        auto.index,
+        # just some arbitrary directory with html files
+        '/usr/share/doc/python3/html/faq',
+    )
+]
+"""
+        cfgdir.mkdir(parents=True)
+        cfg.write_text(stub)
+        logger.info("Created a stub config in %s. Edit it to tune to your linking.", cfg)
+
+
+def config_check(args):
+    cfg = args.config
+    logger = get_logger()
+
+    # TODO add this to HPI
+    logger.info('Checking syntax...')
+    check_call(['python3', '-m', 'compileall', cfg])
+
+    # todo not sure if should be more defensive than check_call here
+    logger.info('Checking type safety...')
+    try:
+        import mypy
+    except ImportError:
+        logger.warning("mypy not found, can't use it to check config!")
+    else:
+        check_call([
+            'python3', '-m', 'mypy',
+            '--namespace-packages',
+            '--color-output', # not sure if works??
+            '--pretty',
+            '--show-error-codes',
+            '--show-error-context',
+            '--check-untyped-defs',
+            cfg,
+        ])
+
+    logger.info('Checking runtime errors...')
+    check_call(['python3', cfg])
+
+
+
 def main():
     # TODO longer, literate description?
     from .common import setup_logger
     setup_logger(get_logger(), level=logging.DEBUG)
 
-    F = argparse.ArgumentDefaultsHelpFormatter
-    p = argparse.ArgumentParser(formatter_class=F)
-    # p.add_argument('--hello', required=False, default='YES', help='alala')
+    F = lambda prog: argparse.ArgumentDefaultsHelpFormatter(prog, width=120)
+    p = argparse.ArgumentParser(formatter_class=F) # type: ignore
     subp = p.add_subparsers(dest='mode', )
     ep = subp.add_parser('index', help='Create/update the link database', formatter_class=F)
-    ep.add_argument('--config', type=Path, default=Path('config.py'))
+    ep.add_argument('--config', type=Path, default=default_config_path(), help='Config path')
     # TODO use some way to override or provide config only via cmdline?
-    ep.add_argument('--intermediate', required=False)
+    ep.add_argument('--intermediate', required=False, help="Used for development, you don't need it")
 
-    sp = subp.add_parser('serve', help='Serve a link database', formatter_class=F)
+    sp = subp.add_parser('serve', help='Serve a link database', formatter_class=F) # type: ignore
     server.setup_parser(sp)
 
     ap = subp.add_parser('demo', help='Demo mode: index and serve a directory in single command', formatter_class=F)
     # TODO use docstring or something?
-    ap.add_argument('--port', type=str, help='Port to serve. If omitted, will only create the index without serving.', required=False)
-    ap.add_argument('--config', type=Path, required=False, help='Config to run against. If omitted, will use empty config')
+    #
+
+    ap.add_argument('--port', type=str, default='13131'              , help='Port to serve on')
+    ap.add_argument('--no-serve', action='store_false', dest='server', help='Pass to only index without running server')
+    ap.add_argument('--config', type=Path, required=False            , help='Config to run against. If omitted, will use empty base config')
     ap.add_argument(
         '--as',
         choices=list(sorted(demo_sources().keys())),
@@ -152,6 +229,15 @@ def main():
     isp = subp.add_parser('install-server', help='Install server as a systemd service (for autostart)', formatter_class=F)
     install_server.setup_parser(isp)
 
+    cp = subp.add_parser('config', help='Config management')
+    scp = cp.add_subparsers()
+    ccp = scp.add_parser('check', help='Check config')
+    ccp.set_defaults(func=config_check)
+    ccp.add_argument('--config', type=Path, default=default_config_path(), help='Config path')
+
+    icp = scp.add_parser('create', help='Create user config')
+    icp.set_defaults(func=config_create)
+
     args = p.parse_args()
 
     # TODO is there a way to print full help? i.e. for all subparsers
@@ -171,8 +257,10 @@ def main():
             server.run(args)
         elif args.mode == 'demo':
             do_demo(index_as=getattr(args, 'as'), params=args.params, port=args.port, config_file=args.config)
-        elif args.mode == 'install-server':
+        elif args.mode == 'install-server': # todo rename to 'autostart' or something?
             install_server.install(args)
+        elif args.mode == 'config':
+            args.func(args)
         else:
             raise AssertionError(f'unexpected mode {args.mode}')
 
