@@ -7,15 +7,18 @@ and
  https://twitter.com/demarionunn/status/928409560548769792
 are same content, but you can't tell that by URL equality. Even canonical urls are different!
 
-Also some experiments to establish 'links' hierarchy.
+Also some experiments to establish 'URL hierarchy'.
 """
+# TODO eh?? they fixed mobile.twitter.com?
+
 import re
 import typing
-from typing import Iterable, NamedTuple, Set, Optional, List
+from typing import Iterable, NamedTuple, Set, Optional, List, Sequence, Union, Tuple, Dict
 
 import urllib.parse
 from urllib.parse import urlsplit, parse_qsl, urlunsplit, parse_qs, urlencode, SplitResult
 
+
 # this has some benchmark, but quite a few librarires seem unmaintained, sadly
 # I guess i'll stick to default for now, until it's a critical bottleneck
 # https://github.com/commonsearch/urlparse4
@@ -85,6 +88,11 @@ default_qremove = {
     'utf8',
 }
 
+default_qkeep = {
+    # ok, various BBS have it, hackernews has it etc?
+    # hopefully it's a reasonable one to keep..
+    'id',
+}
 
 # TODO perhaps, decide if fragment is meaningful (e.g. wiki) or random sequence of letters?
 class Spec(NamedTuple):
@@ -93,29 +101,28 @@ class Spec(NamedTuple):
     fkeep  : bool = False
 
     def keep_query(self, q: str) -> bool:
-        # by default drop all, only do something special in case of specs present
-        # it's better choice for default since if it's too unified user would notice it, but not vice versa
-        if self.qkeep is None and self.qremove is None:
-            return False
-
+        qkeep   = default_qkeep  .union(self.qkeep or {})
         qremove = default_qremove.union(self.qremove or {})
+        # I suppose 'remove' is only useful for logging. we remove by default anyway
 
         keep = False
         remove = False
-        # pylint: disable=unsupported-membership-test
-        if self.qkeep is not None and q in self.qkeep:
+        if qkeep is not None and q in qkeep:
             keep = True
         # pylint: disable=unsupported-membership-test
         if q in qremove:
             remove = True
         if keep and remove:
+            # keep wins?
             return True # TODO need a warning
         if keep:
             return True
         if remove:
             return False
-        return True
-        # TODO basically, at this point only qremove matters
+
+        # by default drop all
+        # it's a better default, since if it's *too* unified, the user would notice it. but not vice versa!
+        return False
 
     @classmethod
     def make(cls, **kwargs):
@@ -168,11 +175,12 @@ specs = {
     ),
     'physicstravelguide.com': S(fkeep=True), # TODO instead, pass fkeep marker object for shorter spec?
     'wikipedia.org': S(fkeep=True),
-    'scottaaronson.com'  : S(qkeep={'p'}, qremove={}, fkeep=True),
-    'urbandictionary.com': S(qkeep={'term'}, qremove={}),
-    'ycombinator.com'    : S(qkeep={'id'}, qremove={}),
-    'play.google.com'    : S(qkeep={'id'}, qremove={}),
-    'answers.yahoo.com'  : S(qkeep={'qid'}, qremove={}),
+    'scottaaronson.com'  : S(qkeep={'p'}, fkeep=True),
+    'urbandictionary.com': S(qkeep={'term'}),
+    'ycombinator.com'    : S(qkeep={'id'}),
+    'play.google.com'    : S(qkeep={'id'}),
+    'answers.yahoo.com'  : S(qkeep={'qid'}),
+    'ubuntuforums.org'   : S(qkeep={'t'}),
 }
 
 _def_spec = S()
@@ -211,6 +219,7 @@ def _quote_path(path: str) -> str:
     return '/'.join(nparts)
 
 
+# TODO wtf is it doing???
 def _prenormalise(url: str) -> str:
     # meh..
     url = re.sub(r'google\..*/amp/s/', '', url)
@@ -233,10 +242,14 @@ def transform_split(split: SplitResult):
 
     fragment = split.fragment
 
-
     ID   = r'(?P<id>[^/]+)'
     REST = r'(?P<rest>.*)'
-    rules = {
+
+    Left = Union[str, Sequence[str]]
+    Right = Tuple[str, str, str]
+    # the idea is that we can unify certain URLs here and map them to the 'canonical' one
+    # this is a dict only for grouping but should be a list really.. todo
+    rules: Dict[Left, Right] = {
         # TODO m. handling might be quite common
         # f'm.youtube.com/{REST}': ('youtube.com', '{rest}'),
         (
@@ -245,6 +258,10 @@ def transform_split(split: SplitResult):
         ) : ('youtube.com', '/watch', 'v={id}'),
         # TODO wonder if there is a better candidate for canonical video link?
         # {DOMAIN} pattern? implicit?
+        (
+            'twitter.com/home',
+            'twitter.com/explore',
+        ) : ('twitter.com', '', ''),
     }
 
     def iter_rules():
@@ -276,6 +293,43 @@ def transform_split(split: SplitResult):
     return netloc, path, qparts, fragment
 
 
+
+def myunsplit(domain: str, path: str, query: str, fragment: str) -> str:
+    uns = urlunsplit((
+        '', # dummy protocol
+        domain,
+        path,
+        query,
+        fragment,
+    ))
+    uns = try_cutl('//', uns)  # // due to dummy protocol
+    return uns
+
+
+
+#
+# r'web.archive.org/web/\d+/
+# def canonify_simple(url: str) -> Optional[str]:
+#     '''
+#     Experiment with simply using regexes for normalising, as close to regular python as it gets
+#     '''
+#     # - using named groups for comments?
+
+#     # TODO to be fair, archive.org is a 'very' special case..
+#     regexes = [
+#         r'web.archive.org/web/(?<timestamp>\d+)/' # TODO what about 'rest'?
+#     ]
+#     for re in regexes:
+
+def handle_archive_org(url: str) -> Optional[str]:
+    are = r'web.archive.org/web/(?P<timestamp>\d+)/(?P<rest>.*)'
+    m = re.fullmatch(are, url)
+    if m is None:
+        return None
+    else:
+        return m.group('rest')
+
+
 # TODO ok, I suppose even though we can't distinguish + and space, likelihood of them overlapping in normalised url is so low, that it doesn't matter much
 # TODO actually, might be easier for most special charaters
 def canonify(url: str) -> str:
@@ -286,6 +340,8 @@ def canonify(url: str) -> str:
         # TODO didn't really get difference from urlparse
         parts = urlsplit(url)
     except Exception as e:
+        # TODO not sure about the exception...
+        # TODO return it instead of raising
         raise CanonifyException(url) from e
 
     # TODO move to prenormalise?
@@ -296,12 +352,18 @@ def canonify(url: str) -> str:
         except Exception as e:
             raise CanonifyException(url) from e
 
+    # meh. figure out what's up with transform_split
+    no_protocol = parts.netloc + parts.path + parts.query + parts.fragment
+
+    res = handle_archive_org(no_protocol)
+    if res is not None:
+        assert len(res) < len(no_protocol) # just a paranoia to avoid infinite recursion...
+        return canonify(res)
 
     domain, path, qq, _frag = transform_split(parts)
 
     spec = get_spec(domain)
 
-
     # TODO FIXME turn this logic back on?
     # frag = parts.fragment if spec.fkeep else ''
     frag = ''
@@ -314,285 +376,10 @@ def canonify(url: str) -> str:
 
     path = _quote_path(path)
 
-    uns = urlunsplit((
-        '',
-        domain,
-        path,
-        query,
-        frag,
-    ))
-
-    uns = try_cutl('//', uns)  # // due to dummy protocol
+    uns = myunsplit(domain, path, query, frag)
     uns = try_cutr('/', uns) # not sure if there is a better way
     return uns
 
-# TODO should actually understand 'sequences'?
-# e.g.
-# https://www.scottaaronson.com/blog/?p=3167#comment-1731882 is kinda hierarchy of scottaaronson.com, post 3167 and comment 1731882
-# but when working with it from server, would be easier to just do multiple queries I guess..
-# https://www.scottaaronson.com/blog/?p=3167 is kind ahierarchy of scottaaronson.com ; 
-
-
-import pytest # type: ignore
-
-
-
-# https://youtu.be/iCvmsMzlF7o
-
-@pytest.mark.parametrize('url,expected', [
-    ( "https://www.youtube.com/watch?v=1NHbPN9pNPM&index=63&list=WL&t=491s"
-    , "youtube.com/watch?list=WL&v=1NHbPN9pNPM" # TODO not so sure about &t, it's sort of useful
-    ),
-    ( "youtube.com/watch?v=wHrCkyoe72U&feature=share&time_continue=6"
-    , "youtube.com/watch?v=wHrCkyoe72U"
-    ),
-
-    ( "youtube.com/embed/nyc6RJEEe0U?feature=oembed"
-    , "youtube.com/watch?v=nyc6RJEEe0U"
-    ),
-
-    # TODO hmm. ordering?
-    ( 'https://youtu.be/iCvmsMzlF7o?list=WL'
-    , 'youtube.com/watch?list=WL&v=iCvmsMzlF7o'
-    ),
-
-    # TODO can even be like that or contain timestamp (&t=)
-    # TODO warn if param already present? shouldn't happen..
-
-    # TODO could be interesting to do automatic rule extraction by querying one represnetative and then extracting canonical
-
-    # TODO national domains don't matter for youtube
-
-    # [*, 'youtube', ANY_DOMAIN] / 'embed' -> 'youtube.com/watch'
-    # TODO use regex backrefs?
-    #
-
-    ( "m.youtube.com/watch?v=Zn6gV2sdl38"
-    , "youtube.com/watch?v=Zn6gV2sdl38"
-    ),
-
-    # ( "https//youtube.com/playlist?list=PLeOfc0M-50LmJtZwyOfw6aVopmIbU1t7t"
-    # , "youtube.com/playlist?list=PLeOfc0M-50LmJtZwyOfw6aVopmIbU1t7t"
-    # ),
-    # TODO perhaps it should result in video link + sibling link?
-    # when exploring other people's playlists this could be quite useful?
-
-    # ( "https://www.youtube.com/watch?v=1NHbPN9pNPM&index=63&list=WL&t=491s"
-    # , "youtube.com/watch?v=1NHbPN9pNPM&list=WL" # TODO not so sure about &t, it's sort of useful
-    # ),
-    # TODO
-    # youtube.com/user/magauchsein/playlists?sort=dd&view=50&shelf_id=14
-    # youtube.com/user/TheChemlife/videos?view=0&sort=p&flow=grid
-])
-def test_youtube(url, expected):
-    assert canonify(url) == expected
-
-
-@pytest.mark.parametrize('url, expected', [
-    ( 'https://www.reddit.com/r/firefox/comments/bbugc5/firefox_bans_free_speech_commenting_plugin/?ref=readnext'
-    , 'reddit.com/r/firefox/comments/bbugc5/firefox_bans_free_speech_commenting_plugin',
-    ),
-
-    ( 'https://www.reddit.com/r/selfhosted/comments/8j8mo3/what_are_you_self_hosting/dz19gh9/?utm_content=permalink&utm_medium=user&utm_source=reddit&utm_name=u_karlicoss'
-    , 'reddit.com/r/selfhosted/comments/8j8mo3/what_are_you_self_hosting/dz19gh9',
-    )
-    # TODO hmm. parent relationship can just rely on urls for reddit
-    # just need to support it in server I suppose
-
-    # TODO search queries?
-    # https://www.reddit.com/search?q=AutoValue
-
-    # TODO def need better markdown handling
-    # https://reddit.com/r/intj/comments/cmof04/me_irl/ew4a3dw/][    Me_irl]  
-    # reddit.com/r/intj/comments/cmof04/me_irl/ew4a3dw/%5D%5BMe_irl%5D
-
-
-
-])
-def test_reddit(url, expected):
-    assert canonify(url) == expected
-
-@pytest.mark.parametrize("url,expected", [
-    # TODO FIXME fragment handling
-    # ( "https://www.scottaaronson.com/blog/?p=3167#comment-1731882"
-    # , "scottaaronson.com/blog/?p=3167#comment-1731882"
-    # ),
-
-
-    # TODO FIXME fragment handling
-    # ( "https://en.wikipedia.org/wiki/tendon#cite_note-14"
-    # , "en.wikipedia.org/wiki/tendon#cite_note-14"
-    # ),
-
-    # TODO FIXME fragment handling
-    # ( "https://physicstravelguide.com/experiments/aharonov-bohm#tab__concrete"
-    # , "physicstravelguide.com/experiments/aharonov-bohm#tab__concrete"
-    # ),
-
-    ( "https://github.com/search?o=asc&q=track&s=stars&type=Repositories"
-    , "github.com/search?q=track"
-    ),
-    ( "https://80000hours.org/career-decision/article/?utm_source=The+EA+Newsletter&utm_campaign=04ca3c2244-EMAIL_CAMPAIGN_2019_04_03_04_26&utm_medium=email&utm_term=0_51c1df13ac-04ca3c2244-318697649"
-    , "80000hours.org/career-decision/article"
-    ),
-    ( "https://www.facebook.com/photo.php?fbid=24147689823424326&set=pcb.2414778905423667&type=3&theater"
-    , "facebook.com/photo.php?fbid=24147689823424326"
-    ),
-    ( "https://play.google.com/store/apps/details?id=com.faultexception.reader&hl=en"
-    , "play.google.com/store/apps/details?id=com.faultexception.reader"
-    ),
-    # TODO it also got &p= parameter, which refers to page... not sure how to handle this
-    # news.ycombinator.com/item?id=15451442&p=2
-    ( "https://news.ycombinator.com/item?id=12172351"
-    , "news.ycombinator.com/item?id=12172351"
-    ),
-    ( "https://urbandictionary.com/define.php?term=Belgian%20Whistle"
-    , "urbandictionary.com/define.php?term=Belgian%20Whistle"
-    ),
-    ( "https://en.wikipedia.org/wiki/Dinic%27s_algorithm"
-    , "en.wikipedia.org/wiki/Dinic%27s_algorithm"
-    ),
-
-    ( "zoopla.co.uk/to-rent/details/42756337#D0zlBWeD4X85odsR.97"
-    , "zoopla.co.uk/to-rent/details/42756337"
-    ),
-
-    ( "withouthspec.co.uk/rooms/16867952?guests=2&adults=2&location=Berlin%2C+Germany&check_in=2017-08-16&check_out=2017-08-20"
-    , "withouthspec.co.uk/rooms/16867952"
-    ),
-
-    ( "amp.theguardian.com/technology/2017/oct/09/mark-zuckerberg-facebook-puerto-rico-virtual-reality"
-    , "theguardian.com/technology/2017/oct/09/mark-zuckerberg-facebook-puerto-rico-virtual-reality",
-    ),
-
-    ( "https://answers.yahoo.com/question/index?qid=20071101131442AAk9bGp"
-    , "answers.yahoo.com/question/index?qid=20071101131442AAk9bGp"
-    ),
-    ( "flowingdata.com/2010/12/14/10-best-data-visualization-projects-of-the-year-%e2%80%93-2010"
-    , "flowingdata.com/2010/12/14/10-best-data-visualization-projects-of-the-year-%E2%80%93-2010"
-    ),
-    ( "flowingdata.com/2010/12/14/10-best-data-visualization-projects-of-the-year-–-2010"
-    , "flowingdata.com/2010/12/14/10-best-data-visualization-projects-of-the-year-%E2%80%93-2010"
-    ),
-
-    ( "https://spoonuniversity.com/lifestyle/marmite-ways-to-eat-it&usg=AFQjCNH4s1SOEjlpENlfPV5nuvADZpSdow"
-    , "spoonuniversity.com/lifestyle/marmite-ways-to-eat-it"
-    ),
-
-    ( 'https://google.co.uk/amp/s/amp.reddit.com/r/androidapps/comments/757e2t/swiftkey_or_gboard'
-    , 'reddit.com/r/androidapps/comments/757e2t/swiftkey_or_gboard'
-    ),
-
-    # should sort query params
-    ( 'https://www.youtube.com/watch?v=hvoQiF0kBI8&list=WL&index=2'
-    , 'youtube.com/watch?list=WL&v=hvoQiF0kBI8',
-    ),
-    ( 'https://www.youtube.com/watch?list=WL&v=hvoQiF0kBI8&index=2'
-    , 'youtube.com/watch?list=WL&v=hvoQiF0kBI8',
-    )
-
-    # ( "gwern.net/DNB+FAQ"
-    # , "TODO" # ???
-    # ),
-
-    # TODO shit. is that normal??? perhaps need to manually move fragment?
-    # SplitResult(scheme='https', netloc='unix.stackexchange.com', path='/questions/171603/convert-file-contents-to-lower-case/171708', query='', fragment='171708&usg=AFQjCNEFCGqCAa4P4Zlu2x11bThJispNxQ')
-    # ( "https://unix.stackexchange.com/questions/171603/convert-file-contents-to-lower-case/171708#171708&usg=AFQjCNEFCGqCAa4P4Zlu2x11bThJispNxQ"
-    # , "unix.stackexchange.com/questions/171603/convert-file-contents-to-lower-case/171708#171708"
-    # )
-])
-def test(url, expected):
-    assert canonify(url) == expected
-    # TODO github queries
-# github.com/search?l=Python&q=reddit+backup
-# github.com/search?p=3&q=ipynb+language%3AHaskell
-# github.com/search?q=kobo+ExtraData
-# github.com/search?q=what-universal-human-experiences-are-you-missing-without-realizing-it
-
-    # TODO git+https://github.com/expectocode/telegram-export@master
-    # TODO  again, for that actually sequence would be good...
-
-    # TODO "https://twitter.com/search?q=pinboard search&src=typd"
-
-    # TODO https://www.zalando-lounge.ch/#/
-    # TODO m.facebook.com
-    # TODO         [R('^(youtube|urbandictionary|tesco|scottaaronson|answers.yahoo.com|code.google.com)') , None],
-
-
-
-    # TODO
-# amazon.co.uk/gp/offer-listing/B00525XKL4/ref=dp_olp_new
-# amazon.co.uk/gp/offer-listing/B00525XKL4/ref=olp_twister_child
-
-    # TODO 
-    # en.wikipedia.org/wiki/S&P_500_Index
-
-
-    # TODO
-    # google.co.uk/maps/place/Hackney+Bureau/@51.5293789,-0.0527919,16.88z/data=!bla-bla!-bla
-
-
-    # TODO 
-    # perhaps, disable utf8 everywhere?
-    # github.com/search?utf8=%E2%9C%93&q=%22My+Clippings.txt%22
-
-    # TODO FIXME fragment handling
-    # ( "https://www.scottaaronson.com/blog/?p=3167#comment-1731882"
-    # , "scottaaronson.com/blog/?p=3167#comment-1731882"
-    # ),
-
-@pytest.mark.parametrize("urls", [
-    {
-        "launchpad.net/ubuntu/%2Bsource/okular",
-        "launchpad.net/ubuntu/+source/okular",
-    },
-    {
-        "flowingdata.com/2010/12/14/10-best-data-visualization-projects-of-the-year-–-2010",
-        "flowingdata.com/2010/12/14/10-best-data-visualization-projects-of-the-year-%e2%80%93-2010",
-        "https://flowingdata.com/2010/12/14/10-best-data-visualization-projects-of-the-year-%e2%80%93-2010/&usg=AFQjCNEZsEGz9rqpWqlFXR5Tc7pkCKY5sQ",
-    },
-])
-def test_same_norm(urls):
-    urls = list(sorted(urls))
-    u0 = urls[0]
-    c0 = canonify(u0)
-    for u in urls[1:]:
-        c = canonify(u)
-        assert c0 == c, f'Expected {u0} and {u} to be same canonically; got {c0} and {c} instead'
-
-def test_error():
-    # canonify('  +74Zo535, fewfwf@gmail.com') # -- apparently was patched in some python3.7 versions
-    with pytest.raises(CanonifyException):
-        # borrowed from https://bugs.mageia.org/show_bug.cgi?id=24640#c7
-        canonify('https://example.com\uFF03@bing.com')
-
-
-# TODO chrome-extension://fdpohaocaechififmbbbbbknoalclacl ??
-# /L/data/wereyouhere/intermediate  ✔  rg 'orig_url.*#' 20190519090753.json | grep -v zoopla | grep -v 'twitter' | grep -v youtube
-
-def main():
-    import argparse
-    p = argparse.ArgumentParser()
-    p.add_argument('input', nargs='?')
-    p.add_argument('--human', action='store_true')
-    p.add_argument('--groups', action='store_true')
-    p.add_argument('--domains', action='store_true')
-    args = p.parse_args()
-
-    it: Iterable[str]
-    if args.input is None:
-        import sys
-        it = sys.stdin
-    else:
-        it = [args.input]
-
-    if args.groups:
-        groups(it, args)
-    if args.domains:
-        domains(it)
-    else:
-        display(it, args)
-
 
  # TODO wonder if lisp could be convenient for this. lol
 TW_PATTERNS = [
@@ -616,6 +403,7 @@ TW_PATTERNS = [
     r'twitter.com/U/lists/L(/.*)?',
     r'twitter.com/i/web/status/S',
     r'twitter.com',
+    r'twitter.com/home',
     r'tweetdeck.twitter.com',
     r'twitter.com/U/(media|photo|followers|following)',
     # r'mobile.twitter.com/.*',
@@ -802,7 +590,7 @@ PATTERNS = {
 }
 
 
-def get_patterns():
+def get_patterns():  # pragma: no cover
     def repl(p, dct):
         for k, v in dct.items():
             p = p.replace(k, v)
@@ -825,7 +613,7 @@ def get_patterns():
     return {k: list(handle(v)) for k, v in PATTERNS.items()}
 
 
-def domains(it):
+def domains(it): # pragma: no cover
     from collections import Counter
     c: typing.Counter[str] = Counter()
     for line in it:
@@ -843,7 +631,7 @@ def domains(it):
     pprint(c.most_common(20))
 
 
-def groups(it, args):
+def groups(it, args): # pragma: no cover
     all_pats = get_patterns()
 
     from collections import Counter
@@ -902,7 +690,9 @@ def groups(it, args):
     pprint(uc)
 
 
-def display(it, args): # TODO better name?
+
+def display(it, args): # pragma: no cover
+    # TODO better name?
     import difflib
     # pylint: disable=import-error
     from termcolor import colored as C # type: ignore
@@ -959,8 +749,38 @@ def display(it, args): # TODO better name?
             stdout.write(f'{org_}\n{can_}\n---\n')
 
 
+def main(): # pragma: no cover
+    import argparse
+    p = argparse.ArgumentParser(epilog='''
+- sqlite3 promnesia.sqlite 'select distinct orig_url from visits' | cannon.py --domains
+
+- running comparison
+  sqlite3 promnesia.sqlite 'select distinct orig_url from visits where norm_url like "%twitter%" order by orig_url' | src/promnesia/cannon.py
+''', formatter_class=lambda prog: argparse.RawTextHelpFormatter(prog, width=100) # type: ignore
+    )
+    p.add_argument('input', nargs='?')
+    p.add_argument('--human', action='store_true')
+    p.add_argument('--groups', action='store_true')
+    p.add_argument('--domains', action='store_true', help='useful for counting number of URLs by domains')
+    args = p.parse_args()
+
+    it: Iterable[str]
+    if args.input is None:
+        import sys
+        it = sys.stdin
+    else:
+        it = [args.input]
+
+    if args.groups:
+        groups(it, args)
+    if args.domains:
+        domains(it)
+    else:
+        display(it, args)
+
+
 if __name__ == '__main__':
-    main()
+    main() # pragma: no cover
 
 # TODO hmm, it's actually sort of fingerprinter... so maybe that's what I should call it
 
@@ -978,9 +798,6 @@ if __name__ == '__main__':
 
 
 # TODO for debugging, have special mode that only uses query param trimming one by one
-
-# TODO running comparison:
-# sqlite3 /L/data/promnesia/promnesia.sqlite 'select distinct orig_url from visits where norm_url like "%twitter%" order by orig_url' | src/promnesia/cannon.py
-
-# TODO sqlite3 /L/data/promnesia/promnesia.sqlite 'select orig_url from visits' | src/promnesia/cannon.py --domains
 # TODO show percents?
+
+# TODO ugh. need coverage...
