@@ -1,30 +1,55 @@
 from pathlib import Path
-from typing import List, Optional, Union, NamedTuple
+from typing import List, Optional, Union, NamedTuple, Iterable
+import importlib
 import importlib.util
 import warnings
 
 import pytz
 
-from .common import PathIsh, get_tmpdir
+from .common import PathIsh, get_tmpdir, appdirs, default_output_dir
+from .common import Res, Source
 
 
 class Config(NamedTuple):
-    OUTPUT_DIR: PathIsh
     # TODO remove default from sources once migrated
     SOURCES: List = []
-    INDEXERS: List = []
+
+    # if not specified, uses user data dir
+    OUTPUT_DIR: Optional[PathIsh] = None
+
     CACHE_DIR: Optional[PathIsh] = None
     FILTERS: List[str] = []
+    #
+    # NOTE: INDEXERS is deprecated, use SOURCES instead
+    INDEXERS: List = []
 
     @property
-    def sources(self):
-        if self.INDEXERS is not None:
+    def sources(self) -> Iterable[Res[Source]]:
+        idx = self.INDEXERS
+
+        if len(self.INDEXERS) > 0:
             warnings.warn("'INDEXERS' is deprecated. Please use 'SOURCES'!", DeprecationWarning)
 
-        res = self.SOURCES or self.INDEXERS
-        # TODO enable it?
-        # assert len(res) > 0, "Expected some sources"
-        return res
+        raw = self.SOURCES + self.INDEXERS
+
+        if len(raw) == 0:
+            raise RuntimeError("Please specify SOURCES in the config! See https://github.com/karlicoss/promnesia#setup for more information")
+
+
+        for r in raw:
+            if isinstance(r, str):
+                # must be a raw module name?
+                try:
+                    r = importlib.import_module(r)
+                except ModuleNotFoundError as e:
+                    yield e
+                    continue
+
+            if isinstance(r, Source):
+                yield r
+            else:
+                # kind of last resort measure..
+                yield Source(r)
 
     @property
     def cache_dir(self) -> Path:
@@ -35,9 +60,15 @@ class Config(NamedTuple):
         res.mkdir(exist_ok=True) # TODO not sure about parents=True
         return res
 
+    # TODO make this optional, default to .cache or something?
+    # TODO also tmp dir -- perhaps should be in cache or at least possible to specify in config? not sure if useful
     @property
     def output_dir(self) -> Path:
-        return Path(self.OUTPUT_DIR)
+        odir = self.OUTPUT_DIR
+        if odir is not None:
+            return Path(odir)
+        else:
+            return default_output_dir()
 
 
 instance: Optional[Config] = None
