@@ -78,7 +78,6 @@ def adhoc_config(tmp_path):
 
     try:
         config.instance = config.Config(
-            INDEXERS=[],
             OUTPUT_DIR=tdir,
             CACHE_DIR=cdir,
         )
@@ -191,6 +190,10 @@ def test_plaintext_path_extractor():
         'http://what.about.this.link',
     }
 
+    [wa] = [v for v in visits if v.orig_url == 'http://what.about.this.link']
+    f2 = Path(tdata('custom')) / 'file2.txt'
+    assert wa.locator.href == f'editor://{f2}:3' # occurs line 3
+
 # TODO perhaps it belongs to canonify?
 def test_normalise():
     import promnesia.sources.shellcmd as custom_gen
